#==============================================================================
# package      DE_EPAGES::Presentation::UI::Group
# state        private
#==============================================================================
package DE_EPAGES::Customer::UI::Group;
use base DE_EPAGES::Presentation::UI::Group;

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias Transaction);
use DE_EPAGES::Core::API::PerlTools qw (unique isValid);
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );

sub AddUsersFromTray {
    my $self = shift;
    my ($Servlet) = @_;

    my $Group = $Servlet->object;
    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    my $aNewCustomers = $Tray->objectsByClass(LoadClassByAlias('Customer'));

    Transaction( sub {
        foreach my $aNewUsers (map { $_->get('Users') } @$aNewCustomers) {
            $Group->addMember($_) foreach grep { not $Group->existsMember($_) } @$aNewUsers;
        }
    });
    return $self->SUPER::AddUsersFromTray($Servlet);
}

sub UnityCustomerGroupDeleteGlobalValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    my $CustomerGroup = $Servlet->object;
    my $Class = LoadClassByAlias('GlobalValueDiscount');

    my $PriceList = @{$CustomerGroup->get('PriceLists')}[0];
    my $FormValues  = $Servlet->form->form($PriceList, 'DeleteGlobalValueDiscounts');

    my @ListObjects;
    foreach my $GlobalValueDiscount (@{$FormValues->{'GlobalValueDiscounts'}}) {
        if($Class->existsObject($GlobalValueDiscount->{'ListObjectID'})) {
            push @ListObjects, $Class->loadObject($GlobalValueDiscount->{'ListObjectID'});
        }
    }

    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        $Servlet->vars('DeleteObjects', [ map { $_->tleHash } @ListObjects]);
        $Servlet->vars('DELETE',1);
        return;
    } else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    foreach my $ListObject (@ListObjects) {
        TriggerHook('UI_DeleteUnityCustomerGroupGlobalValueDiscounts', $ListObject);
    }
}

sub UnityCustomerGroupSaveGlobalValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    my $CustomerGroup = $Servlet->object;
    my $aPriceLists = $CustomerGroup->get('PriceLists');

    # In case we have Unity CustomerGroups without pricelists, we need first to create a new pricelist for that group
    unless (scalar @{$aPriceLists}) {
        push @$aPriceLists, _CreateUnityPriceListForCustomerGroup($CustomerGroup);
    }

    my $PriceList = $aPriceLists->[0];
    my $Class       = LoadClassByAlias('GlobalValueDiscount');

    # new value discounts
    if ( $Servlet->params('NewDiscount') !~ /^\s*$/ or $Servlet->params('NewDiscountAbsolute') !~ /^\s*$/ or $Servlet->params('NewValue') !~ /^\s*$/){
        _NewGlobalValueDiscounts($Servlet, $PriceList);
    }

    # save value discounts
    my $FormValues = $Servlet->form->form($PriceList, 'UnitySaveGlobalValueDiscounts');
    my $Index = 0;
    foreach my $Entry (@{$FormValues->{'GlobalValueDiscounts'}}) {
        my $GlobalValueDiscount = $Class->loadObject($Entry->{'GlobalValueDiscountID'});
        # security check
        die 'GlobalValueDiscount from different PriceList' if $PriceList->id != $GlobalValueDiscount->get('PriceList')->id;

        if (defined  $GlobalValueDiscount) {
            if (defined $Entry->{'Discount'} || defined $Entry->{'DiscountAbsolute'}) {
                # test absolute discount > interval
                if (defined $Entry->{'DiscountAbsolute'}) {
                    my $DiscountAbsolute = $Entry->{'DiscountAbsolute'};
                    $Servlet->form->executeFormError({
                        'Reason'=> 'WRONG_DISCOUNT',
                        'Name'  => 'DiscountAbsolute',
                        'Form'  => 'SaveGlobalValueDiscounts',
                        'LoopName' => 'GlobalValueDiscounts',
                        'Index' => $Index,
                    }) if $DiscountAbsolute > $Entry->{'Value'};
                }
                #test for duplicated edit values
                my @Values;
                foreach my $GlobalValue (@{$FormValues->{'GlobalValueDiscounts'}}) {
                    push @Values, $GlobalValue->{Value} * 1 if defined  $GlobalValue->{Value};
                }
                my @ExistingGlobalValueDiscounts = map { $_->get('Value') * 1 } @{ $PriceList->get('GlobalValueDiscounts') };
                my @UniqueArr = unique(@Values);
                my $Duplicates = (scalar(@Values) == scalar(@UniqueArr)) ? 0 : 1;
                if ($Duplicates) {
                    $Servlet->form->executeFormError({
                        'Reason'=> 'INTERVAL_EXISTS',
                        'Name'  => 'Value',
                        'Form'  => 'SaveGlobalValueDiscounts',
                        'LoopName' => 'GlobalValueDiscounts',
                        'Index' => _GetDuplicateIndex( \@Values , \@ExistingGlobalValueDiscounts ),
                    });
                }
                delete $Entry->{'Discount'} if defined $Entry->{'Discount'} and $Entry->{'Discount'} !~ /\S/;
                delete $Entry->{'DiscountAbsolute'} if defined $Entry->{'DiscountAbsolute'} and $Entry->{'DiscountAbsolute'} !~ /\S/;
                delete $Entry->{'Value'} if defined $Entry->{'Value'} and $Entry->{'Value'} !~ /\S/;
                $GlobalValueDiscount->set({'Value'=>$Entry->{'Value'}, 'Discount'=>$Entry->{'Discount'}, 'DiscountAbsolute' => $Entry->{'DiscountAbsolute'} });
            } else {
                $GlobalValueDiscount->delete;
            }
        }
        $Index++;
    }
    return;
}

sub _NewGlobalValueDiscounts {
    my ($Servlet, $PriceList) = @_;

    my $Shop        = $PriceList->getSite();
    my $FormValues  = $Servlet->form->form($PriceList, 'UnityNewGlobalValueDiscount');
    my @GlobalValues = @{$FormValues->{'NewGlobalValueDiscounts'}};

    foreach my $hEntry ( @GlobalValues ) {
        next unless isValid($hEntry->{'NewValue'});
        my $Value       = $Shop->formatter($PriceList->get('CurrencyID'))->round($hEntry->{'NewValue'});

        #test for discount or absolute discount
        unless (defined($hEntry->{'NewDiscountAbsolute'}) || defined($hEntry->{'NewDiscount'})) {
            $Servlet->form->addFormError({
                'Reason'=> 'MISSING_DISCOUNT',
                'Name'  => 'NewDiscountAbsolute',
                'Form'  => 'NewGlobalValueDiscount',
            });
            $Servlet->form->addFormError({
                'Reason'=> 'MISSING_DISCOUNT',
                'Name'  => 'NewDiscount',
                'Form'  => 'NewGlobalValueDiscount',
            });
            $Servlet->form->executeFormError();
        }
        #test for duplicated new values
        my @NewValues;
        foreach my $GlobalValue (@GlobalValues){
            push @NewValues, $GlobalValue->{NewValue} if defined  $GlobalValue->{NewValue};
        }
        my @Array_uniq = unique @NewValues;
        my $Duplicates =  (scalar(@NewValues) == scalar(@Array_uniq)) ? 0 : 1;
        if ($Duplicates) {
            $Servlet->form->executeFormError({
                'Reason'=> 'INTERVAL_EXISTS',
                'Name'  => 'NewValue',
                'Form'  => 'NewGlobalValueDiscount',
            });
        }
        #test for existing interval
        my @ExistingGlobalValueDiscounts = grep { $_->get('Value') * 1 == $Value } @{ $PriceList->get('GlobalValueDiscounts') };
        if (scalar @ExistingGlobalValueDiscounts) {
            $Servlet->form->executeFormError({
                'Reason'=> 'INTERVAL_EXISTS',
                'Name'  => 'NewValue',
                'Form'  => 'NewGlobalValueDiscount',
            });
        }
        # test absolute discount > interval
        if (defined $hEntry->{'NewDiscountAbsolute'}) {
            my $NewDiscountAbsolute = $hEntry->{'NewDiscountAbsolute'};
            $Servlet->form->executeFormError({
                'Reason'=> 'WRONG_DISCOUNT',
                'Name'  => 'NewDiscountAbsolute',
                'Form'  => 'NewGlobalValueDiscount',
            }) if $NewDiscountAbsolute > $Value;
        }

        my $Class=LoadClassByAlias('GlobalValueDiscount');
        $Class->insertObject ({
            'PriceList'   => $PriceList,
            'Value'       => $Value,
            'Discount'    => $hEntry->{'NewDiscount'},
            'DiscountAbsolute' => $hEntry->{'NewDiscountAbsolute'},
        });
    }
    return;
}

sub _GetDuplicateIndex {
    my ($aValues, $aExistingGlobalValueDiscounts) = @_;

    my $Index = 0;
    foreach my $Value (@{$aExistingGlobalValueDiscounts}) {
        if ($aValues->[$Index] != $Value) {
            return $Index;
        };
        $Index++;
    }
}

sub _CreateUnityPriceListForCustomerGroup($) {
    my ($CustomerGroup) = @_;

    my $Shop    = $CustomerGroup->getSite();
    my $Folder = $Shop->folder('PriceLists', { 'Position' => 30 });
    my $PriceListAlias = $CustomerGroup->alias;
    my $PriceList;

    if ($Folder->existsChild($PriceListAlias)) {
        $PriceList =  $Folder->child($PriceListAlias);
    } else {
        my $Class = LoadClassByAlias('PriceList');
        my $hPriceList = {
            'Parent'     => $Folder,
            'Alias'      => $PriceListAlias,
            'ValidFrom'  => DateTime->now(),
            'ValidTo'    => DateTime->new('year' => 2999, 'month' => 1, 'day' => 1),
            'CurrencyID' => $Shop->get('CurrencyID'),
        };
        $PriceList = $Class->insertObject($hPriceList);

        # assign unity customergroup to pricelist
        $PriceList->addCustomerGroup($CustomerGroup);

    }
    return $PriceList;
}

1;
