#==============================================================================
# package      DE_EPAGES::GoogleAnalytics::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::GoogleAnalytics::UI::Shop;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;

use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::GoogleAnalytics::API::Table::ShopGoogleAnalyticsAction qw( ExistsShopGoogleAnalyticsAction UpdateShopGoogleAnalyticsAction InsertShopGoogleAnalyticsAction );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#==============================================================================
sub Save {
    my $self    = shift;
    my ($Servlet) = @_;
    my $Shop    = $Servlet->object;

    my $Form = $Servlet->form;
    my $GoogleAnalyticsWebPropertyID = $Form->value('GoogleAnalyticsWebPropertyID');
    if(defined $GoogleAnalyticsWebPropertyID) {
        # GoogleAnalyticsWebPropertyID should only contain regular characters or -
        if ($GoogleAnalyticsWebPropertyID !~ /^[\w\-]*$/) {
            $Form->executeFormError({
                'Name'   => 'GoogleAnalyticsWebPropertyID',
                'Reason' => 'INVALID_CHARACTER'
            });
        } elsif ($GoogleAnalyticsWebPropertyID !~ /^UA-\d+-\d+$/ && $GoogleAnalyticsWebPropertyID ne "") {
            $Form->executeFormError({
                'Name'   => 'GoogleAnalyticsWebPropertyID',
                'Reason' => 'INVALID_UA_FORMAT'
            });
        }
    }

    # standard save
    $self->SUPER::Save($Servlet);

    my $ShopID = $Shop->id;
    my $ahGoogleAnalyticsActions = $Shop->get('GoogleAnalyticsActions');

    foreach my $hGoogleAnalyticsAction (@$ahGoogleAnalyticsActions) {
        my $ActionID = $hGoogleAnalyticsAction->{'Action'}->id;
        InsertShopGoogleAnalyticsAction({'ShopID' => $ShopID, 'ActionID' => $ActionID, 'IsActive' => 1}) unless ExistsShopGoogleAnalyticsAction($ShopID, $ActionID);
    }

    return;
}

#==============================================================================
sub SaveActions {
    my $self    = shift;
    my ($Servlet) = @_;
    my $Shop    = $Servlet->object;

    my $hFormValues = $Servlet->form->form($Shop, 'SaveGoogleAnalyticsActions');
    my @ActiveActionIDs = map {$_->{'IsActive'}} @{$hFormValues->{'GoogleAnalyticsActiveActions'}};

    foreach my $hGoogleAnalyticsAction (@{$hFormValues->{'GoogleAnalyticsActions'}}) {
        $hGoogleAnalyticsAction->{'IsActive'} = ElementOf($hGoogleAnalyticsAction->{'ActionID'}, @ActiveActionIDs) ? 1 : 0 ;
        $hGoogleAnalyticsAction->{'ShopID'}  = $Shop->id;
        ExistsShopGoogleAnalyticsAction($hGoogleAnalyticsAction) ? UpdateShopGoogleAnalyticsAction($hGoogleAnalyticsAction) : InsertShopGoogleAnalyticsAction($hGoogleAnalyticsAction);
    }

    return;
}

#==============================================================================
sub SwitchToUniversalAnalytics {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    unless ($Shop->get('GoogleAnalyticsUseUniversalAnalytics')) {
        $Shop->set({'GoogleAnalyticsUseUniversalAnalytics' => 1});
    }
}

#==============================================================================
sub MBOViewGoogleAnalytics {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_ViewGoogleAnalytics', { 'Servlet' => $Servlet });

    my $Shop = $Servlet->object;

    unless ($Shop->get('GoogleAnalyticsWebPropertyID') or $Shop->get('GoogleAnalyticsUseUniversalAnalytics')) {
        $Shop->set({'GoogleAnalyticsUseUniversalAnalytics' => 1});
    }

    return $self->SUPER::View($Servlet);
}

1;
