#==============================================================================
# package      DE_EPAGES::Paynet::UI::BasketOrder
# state        private
#==============================================================================
package DE_EPAGES::Paynet::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use Digest;
use DE_EPAGES::ExternalPayment::API::Log qw (LogPayment);
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Presentation::API::LocaleHandler qw ( AddLanguageToURL );

sub ViewPaymentPaynet {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object    = $Servlet->object;

    my $Container = $Object->container;
    return unless defined $Container;

    my $Payment = $Container->get('Payment');
    my $PaymentMethod = $Payment->get('PaymentMethod');
    my $ProjectPW = $PaymentMethod->get('ProjectPW');
    return unless defined $ProjectPW;

    my @PaymentParams = (
        {'Name' => 'user_id',    'Value' => $PaymentMethod->get('Paynet_CustomerID')},
        {'Name' => 'project_id', 'Value' => $PaymentMethod->get('Paynet_ProjectID')},
    );

    #sender_holder
    my $BillingAddress = $Object->get('BillingAddress');
    my $UserBillingAddress = $Object->get('User')->get('BillingAddress');
    push(@PaymentParams, {'Name' => 'sender_holder', 'Value' => $BillingAddress->get('DisplayName') // $UserBillingAddress->get('FullName')});

    # sender_account_number, sender_bank_code, sender_country_id unused
    push(@PaymentParams, {'Name' => 'sender_account_number', 'Value' => ''});
    push(@PaymentParams, {'Name' => 'sender_bank_code',      'Value' => ''});
    push(@PaymentParams, {'Name' => 'sender_country_id',     'Value' => ''});

    # amount and currency
    push(@PaymentParams, {'Name' => 'amount',      'Value' => $Payment->get('Amount')});
    push(@PaymentParams, {'Name' => 'currency_id', 'Value' => $Container->get('CurrencyID')});

    my $LanguageID = $Servlet->languageID;
    # reason_1
    if ($Object->class->alias eq 'CustomerOrder') {
        push(@PaymentParams, {'Name' => 'reason_1', 'Value' => ($PaymentMethod->get('Reason1', $LanguageID) // 'Bestell-Nr.')  . ' ' . $Object->alias});
    }
    else {
        push(@PaymentParams, {'Name' => 'reason_1', 'Value' => ($PaymentMethod->get('Reason1', $LanguageID) // 'Refnr.')  . ' ' . $Object->id});
    }

    # reason_2
    my $Customer = $Object->get('Customer');
    if ($Customer){
        push(@PaymentParams, {'Name' => 'reason_2', 'Value' => ($PaymentMethod->get('Reason2', $LanguageID) // 'Kunden-Nr.')  . ' ' . $Customer->alias});
    }
    else {
        push(@PaymentParams, {'Name' => 'reason_2', 'Value' => $UserBillingAddress->get('FullName')});
    }

    # user_variable_0 unused
    push(@PaymentParams, {'Name' => 'user_variable_0', 'Value' => ''});

    # user_variable_1
    push(@PaymentParams, {'Name' => 'user_variable_1', 'Value' => $Object->id});

    # user_variable_2, user_variable_3, user_variable_4 unused
    push(@PaymentParams, {'Name' => 'user_variable_2', 'Value' => ''});
    push(@PaymentParams, {'Name' => 'user_variable_3', 'Value' => ''});
    push(@PaymentParams, {'Name' => 'user_variable_4', 'Value' => ''});

    # user_variable_5
    push(@PaymentParams, {'Name' => 'user_variable_5', 'Value' => $Payment->get('GUID')});

    # calculate key
    my $key = join('|', ((map { $_->{'Value'} } @PaymentParams), $ProjectPW));
    $key = Digest->new('SHA-256')->add($key)->hexdigest;

    my $interfaceVersion = LoadRootObject()->get('Paynet_InterfaceVersion');

    # debugging
    LogPayment('Paynet', 'payment params for payment line item ' . $Payment->get('GUID') . " (hash=$key)",
        [\@PaymentParams, {'interface_version' => $interfaceVersion}]);

    # set TLE variables
    $Servlet->vars('Key', $key);
    $Servlet->vars('PaynetInterfaceVersion', $interfaceVersion);
    $Servlet->vars('PaynetPaymentParams', \@PaymentParams);

    return;
}

#==============================================================================
sub ViewMultiPaymentPaynet {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash);

    $self->ViewPaymentPaynet($Servlet);
    return;
}

sub ViewPaymentPaynetCancel {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrOrder = $Servlet->object;
    my $Shop = $BasketOrOrder->get('Shop');
    my $LineItemPayment = $BasketOrOrder->container()->get("Payment");

    LogPayment('Paynet', 'Payment cancellation: ', $LineItemPayment->id());

    if ($Shop->isUnity()) {
        $LineItemPayment->set({'PaymentErrorCode' => 'Canceled'});
        my $UserLanguageID = $Servlet->user()->get('LanguageID');
        my $URL = AddLanguageToURL($BasketOrOrder->get('WebUrlUnitySSL'), $UserLanguageID, 0);
        $Servlet->logout() if $Servlet->user()->get('IsAnonymous');
        $Servlet->redirect($URL);
    }
}

1;
